/*
 * PM - 2017
 * lab0.c
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "../avr-nokia5110-master/nokia5110.h"
#include "../pwm_music/notes.h"
#include "../DS3231-RTC-AVR-Atmega-Library-master/DS3231.h"
#include "keypad.h"

/// Default volume, value assigned to OCR1B
#define DEFAULT_VOLUME 200
#define ALARM_SET "Alarms are set"
#define NO_ALARM "No alarm set"

#define SET_HOURS "Set hours"
#define SET_MINUTES "Set minutes"

typedef enum {
	STAR_WARS,
	FUR_ELISE,
	JUDE
} Tone;

typedef enum {
	DEFAULT,
	ALARM_TONE,
	ALARM_TIME,
	RINGING,
	SET_TIME
} State;

struct rtc_time {
	uint8_t hour;
	uint8_t min;
	uint8_t sec;
};

struct rtc_date {
	uint8_t wday;
	uint8_t day;
	uint8_t month;
	uint8_t year;
};

typedef struct {
	struct rtc_time time;
	struct rtc_date date;
	Tone alarm_tone;
} Alarm;

// Star Wars
const int starwars[] = 
{
	Ais2,8, Ais2,8, P,16, F3,8, F3,8, P,16, Dis3,16, P,16, D3,16, P,16, C3,16, P,16, Ais3,8,
	Ais3,8, P,16, F3,8, P,16, Dis3,16, P,16, D3,16, P,16, C3,16, P,16, Ais3,8, Ais3,8, P,16,
	F3,8, P,16, Dis3,16, P,16, D3,16, P,16, Dis3,16, P,16, C3,8, C3,8, 
	MUSIC_END
};

// Fur Elise
const int furelise[] = 
{
    e4, 8, d4x, 8, e4, 8, d4x, 8, e4, 8, b3, 8, d4, 8, c4, 8, a3,8, p, 8,
    c3, 8, e3, 8, a3, 8,  b3, 4, p, 8, e3, 8, g3x, 8, b3, 8, c4, 4, p, 8, e3, 8,
    e3, 8, d4x, 8, e4, 8, d4x, 8, e4, 8, b3, 8, d4, 8, c4, 8, a3, 8, p, 8, c3, 8,
    e3, 8, a3, 8, b3, 4, p, 8, e3, 8, c4, 8, b3, 8, a3, 4,
    MUSIC_END
};

// Beatles, Hey Jude
const int Jude[] = {
	G2,8, E2,8, P,16, E2,16, E2,16, G2,16, A2,16, D2,8, P,16, D2,16, E2,16, F2,8, 
	C3,8, C3,16, C3,16, H2,16, G2,16, A2,16, G2,16, F2,16, E2,8, P,16, G2,16, 
	A2,16, A2,8, A2,16, D3,16, C3,16, H2,16, H2,16, C3,16, A2,16, G2,8, P,16, 
	C2,16, D2,16, E2,16, A2,16, A2,16, G2,8,
	MUSIC_END
};

void InitMusic()
{
	// Configure OC1B pin as output

	DDRD |= _BV(DDD4); //OC1B as output

	// timer1 configuration (for PWM)
    TCCR1A |= _BV(COM1B1);  // Clear OC1A/OC1B on compare match

	
	TCCR1B |= _BV(WGM13) 	//mode 8, PWM, Phase and Frequency Correct (TOP value is ICR1)
		   |_BV(CS11); 		//prescaler(8)
}


volatile int alarm_on = 0;
volatile int alarm_set = 1;
volatile char time_str[20];
volatile char date_str[20];
volatile State state = DEFAULT;
volatile int tone_position = 0;
volatile Alarm alarms[3];
volatile int num_alarms = 0;
volatile int nr_rings = 3;
volatile int ringing_alarm_tone = 0;
volatile int ringing_alarm_num = 0;
volatile int total_alarms = 1;

// system data
 struct rtc_time system_time;
 struct rtc_date system_date;

 volatile int alarm_set_minutes = 0;
 volatile int alarm_set_hours = 0;
 volatile int alarm_set_position = 0;
 
 volatile int time_set_minutes = 0;
 volatile int time_set_hours = 0;
 volatile int time_set_position = 0;


void time_to_str() {
	int i;
	for (i = 0; i < 20; i++) {
		time_str[i] = 0;
	}
	char hour[2];
	char minute[2];
	char sec[2];
	
	hour[0] = system_time.hour / 10 + '0';
	hour[1] = system_time.hour % 10 + '0';
	
	minute[0] = system_time.min / 10 + '0';
	minute[1] = system_time.min % 10 + '0';
	
	sec[0] = system_time.sec / 10 + '0';
	sec[1] = system_time.sec % 10 + '0';
	
	for (i = 0; i < 2; i++) {
		time_str[i] = hour[i];
	}
	time_str[2] = ':';
	for (i = 0; i < 2; i++) {
		time_str[i + 3] = minute[i];
	}
	time_str[5] = ':';
	for (i = 0; i < 2; i++) {
		time_str[i + 6] = sec[i];
	}
}

void date_to_str() {
	int i;
	for (i = 0; i < 20; i++) {
		date_str[i] = 0;
	}
	char day[2];
	char month[2];
	char year [2];
	
	day[0] = system_date.day / 10 + '0';
	day[1] = system_date.day % 10 + '0';
	
	month[0] = (system_date.month / 10) % 10 + '0';
	month[1] = (system_date.month % 10) + '0';
	
	year[0] = (system_date.year / 10) % 10 + '0';
	year[1] = system_date.year % 10 + '0';
	
	for (i = 0; i < 2; i++) {
		date_str[i] = day[i];
	}
	date_str[2] = '/';
	for (i = 0; i < 2; i++) {
		date_str[i + 3] = month[i];
	}
	date_str[5] = '/';
	for (i = 0; i < 2; i++) {
		date_str[i + 6] = year[i];
	}
	
}

void set_time(uint8_t hour, uint8_t minute, uint8_t second){
	system_time.hour = hour;
	system_time.min = minute;
	system_time.sec = second;
	time_to_str();
}

void set_date(uint8_t day, uint8_t month, uint8_t year) {
	system_date.wday = 1;
	system_date.day = day;
	system_date.month = month;
	system_date.year = year;
	date_to_str();
}

void PlayMusic( const int* pMusicNotes /** Pointer to table containing music data */,
				uint8_t tempo /** paying tempo from 0 to 100. Higher value = slower playback*/ )
{
	int duration;
	int note;
	int i;
	uint16_t delay = tempo * 1000;

	while( *pMusicNotes )
	{
		note = *pMusicNotes;
		pMusicNotes++;

		duration = *pMusicNotes;
		pMusicNotes++;

		if( p == note )
		{
			//pause, do not generate any sound
			OCR1B = 0;
		}
		else
		{
			//not a pause, generate tone
			OCR1B = DEFAULT_VOLUME;

			//set frequency
			ICR1H = (note >> 8);
			ICR1L = note;
		}
		//wait duration
		for(i=0;i<32-duration;i++)
		{
			_delay_loop_2( delay );
		}
		
		if (alarm_on == 0) {
			break;
		}
	}
	
	//turn off any sound
	OCR1B = 0;
}


void setup_buttons(void) {
	// intrarea butoanelor
	DDRD &= ~(1 << PD0);
	PORTD |= (1 << PD0);
	
	DDRD &= ~(1 << PD1);
	PORTD |= (1 << PD1);
	
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);
	
	DDRC &= ~(1 << PC2);
	PORTC |= (1 << PC2);
	
	DDRC &= ~(1 << PC4);
	PORTC |= (1 << PC4);
	
	DDRB |= 1 << PB5;
	PORTB |= 1 << PB5;
	
	// intrerupere
	PCICR |= (1 << PCIE3) | (1 << PCIE2);
	PCMSK3 |= (1 << PCINT24) | (1 << PCINT25);
	PCMSK2 |= (1 << PCINT18) | (1 << PCINT20);
}




void display_default(void) {
    nokia_lcd_clear();
	if (alarm_set) {
		nokia_lcd_write_string(ALARM_SET, 1);
	} else {
		nokia_lcd_write_string(NO_ALARM, 1);
	}
	nokia_lcd_set_cursor(10, 20);
	char *time = (char *)time_str;
    nokia_lcd_write_string(time, 1);
	nokia_lcd_set_cursor(10, 40);
	char *date = (char *)date_str;
	nokia_lcd_write_string(date, 1);
    nokia_lcd_render();
}

void display_ringing(void) {
	nokia_lcd_clear();
	nokia_lcd_write_string("Ringing", 2);
	nokia_lcd_set_cursor(0, 30);
	nokia_lcd_write_string((char *)time_str, 2);
	nokia_lcd_render();
}

void display_alarm_time(void) {
	nokia_lcd_clear();
	if (alarm_set_position == 0) {
		nokia_lcd_write_string(SET_HOURS, 1);
	} else {
		nokia_lcd_write_string(SET_MINUTES, 1);
	}
	
	nokia_lcd_set_cursor(0, 20);
	char hour[2];
	char minute[2];
	
	hour[0] = alarm_set_hours / 10 + '0';
	hour[1] = alarm_set_hours % 10 + '0';
	
	minute[0] = alarm_set_minutes / 10 + '0';
	minute[1] = alarm_set_minutes % 10 + '0';
	
	char str[10];
	int i;
	
	for (i = 0; i < 10; i++) {
		str[i] = 0;
	}
	
	str[0] = hour[0];
	str[1] = hour[1];
	str[2] = ':';
	str[3] = minute[0];
	str[4] = minute[1];
	
	nokia_lcd_write_string(str, 2);
	nokia_lcd_render();
}

void display_set_time(void){
	nokia_lcd_clear();
	if (time_set_position == 0) {
		nokia_lcd_write_string(SET_HOURS, 1);
	} else {
		nokia_lcd_write_string(SET_MINUTES, 1);
	}
	
	nokia_lcd_set_cursor(0, 20);
	char hour[2];
	char minute[2];
	
	
	hour[0] = time_set_hours / 10 + '0';
	hour[1] = time_set_hours % 10 + '0';
	
	minute[0] = time_set_minutes / 10 + '0';
	minute[1] = time_set_minutes % 10 + '0';
	
	char str[10];
	int i;
	
	for (i = 0; i < 10; i++) {
		str[i] = 0;
	}
	
	str[0] = hour[0];
	str[1] = hour[1];
	str[2] = ':';
	str[3] = minute[0];
	str[4] = minute[1];
	
	nokia_lcd_write_string(str, 2);
	nokia_lcd_render();
}

void display_tone() {
	nokia_lcd_clear();
	if (tone_position == 0)
		nokia_lcd_write_string(">StarWars", 1);
	else
		nokia_lcd_write_string("StarWars", 1);
	nokia_lcd_set_cursor(0, 15);
	if (tone_position == 1)
		nokia_lcd_write_string(">Furelise", 1);
	else
		nokia_lcd_write_string("Furelise", 1);
	nokia_lcd_set_cursor(0, 30);
	if (tone_position == 2)
		nokia_lcd_write_string(">Jude", 1);
	else
		nokia_lcd_write_string("Jude", 1);
	nokia_lcd_render();
}

void triggerAlarm() {
	int ok = 0;
	int i;
	for (i = 0; i < 3; i++) {
		if (alarms[i].time.sec == system_time.sec &&
				alarms[i].time.min == system_time.min &&
				alarms[i].time.hour == system_time.hour &&
				alarm_set)
			ok = 1;
		if (ok) {
			alarm_on = 1;
			state = RINGING;
			ringing_alarm_tone = alarms[i].alarm_tone;
			ringing_alarm_num = i;
			PORTD |= (1 << PD7);
			PORTB &= ~(1 << PB5);
			break;
		}
	}
	
}

void rtc_update() {
	if (state == SET_TIME)
		return;
	DS3231_getTime(&system_time.sec, &system_time.min, &system_time.hour,
                    &system_date.wday,
                    &system_date.day,
                    &system_date.month,
                    &system_date.year);
	time_to_str();
	date_to_str();
	
	triggerAlarm();
}

void rtc_init() {

	DS3231_setTime(system_time.sec, system_time.min, system_time.hour,
                    system_date.wday,
                    system_date.day,
                    system_date.month,
                    system_date.year);
}

void create_alarm(uint8_t hour, uint8_t min, uint8_t sec,
					uint8_t wday, uint8_t day, uint8_t month, uint8_t year,
					Tone tone, Alarm *alarm)
{
	alarm->alarm_tone = tone;
	
	alarm->time.sec = sec;
	alarm->time.min = min;
	alarm->time.hour = hour;
	
	alarm->date.wday = wday;
	alarm->date.day = day;
	alarm->date.month = month;
	alarm->date.year = year;
	
}
						

void finish_alarm(){
	num_alarms = (num_alarms + 1) % 3;
	create_alarm(alarm_set_hours, alarm_set_minutes, 0,
			1, system_date.day, system_date.month, system_date.year,
			(Tone) tone_position, (Alarm *)&alarms[num_alarms]);
	alarm_set = 1;
	total_alarms++;
}

void ring_alarm() {
	switch(ringing_alarm_tone) {
	case 0:
		PlayMusic(starwars, 90);
		break;
	case 1:
		PlayMusic(furelise, 100);
		break;
	default:
		PlayMusic(Jude, 78);
	}
}

void timer0_init()
{
    TCCR0B |= (1 << CS02);
  
    TIMSK0 |= (1 << TOIE0);
	
}

void set_the_alarm(){

	while (1) {
		char first_digit = keyfind();
		char second_digit = keyfind();
		int nr = (first_digit - '0') * 10 + (second_digit - '0');
		if ((nr < 24 && !alarm_set_position) || (nr < 60 && alarm_set_position)) {
			if (!alarm_set_position)
				alarm_set_hours = nr;
			else
				alarm_set_minutes = nr;
			break;
		}
	}
}

void set_the_time(){
	while (1) {
		char first_digit = keyfind();
		char second_digit = keyfind();
		int nr = (first_digit - '0') * 10 + (second_digit - '0');
		if ((nr < 24 && !time_set_position) || (nr < 60 && time_set_position)) {
			if (!time_set_position)
				time_set_hours = nr;
			else
				time_set_minutes = nr;
			break;
		}
	}
}

ISR(TIMER0_OVF_vect)
{
    rtc_update();
}

ISR(PCINT2_vect) {
	
	if (!(PINC & (1 << PC4))) {
		if (state == DEFAULT) {
			state = SET_TIME;
		}
	}
	
	if (!(PINC & (1 << PC2))) {
		
		if (state == ALARM_TIME) {
			state = DEFAULT;
			alarm_set = 1;
			_delay_ms(100);
		}
		
		if (state == ALARM_TONE) {
			state = ALARM_TIME;
			_delay_ms(100);
		}
		

	}
	

	
}

ISR(PCINT3_vect) {
	if (!(PIND & (1 << PD0))) {
		if (state == RINGING) {
			PORTD &= ~(1 << PD7);
			PORTB |= (1 << PB5);
			state = DEFAULT;
			alarm_on = 0;
			total_alarms--;
			if (total_alarms == 0) {
				alarm_set = 0;
			}
		}
		
		if (state == ALARM_TONE) {
			state = DEFAULT;
			tone_position = 0;
		}
		_delay_ms(100);
	}
	
	if (!(PIND & (1 << PD1))) {
		if (state != RINGING && state != ALARM_TIME) {
			if (state != ALARM_TONE) {
				state = ALARM_TONE;
				tone_position = 0;
			} else {
				if (state == ALARM_TONE) {
					tone_position = (tone_position + 1) % 3;
				}
			}
		}
		_delay_ms(100);
	}
}

int main(void) {
	
	
	set_time(15, 32, 5);
	set_date(21, 5, 18);
	create_alarm(15, 32, 10, 1, 21, 5, 18, STAR_WARS, (Alarm *)&alarms[0]);
	
	nokia_lcd_init();
	twi_INIT();
	rtc_init();
	timer0_init();
	setup_buttons();
	InitMusic();

	sei();
	while (1) {
		if (state == DEFAULT)
			display_default();
		if (state == SET_TIME) {
			display_set_time();
			set_the_time();
			time_set_position = 1;
			display_set_time();
			set_the_time();
			display_set_time();
			time_set_position = 0;
			system_time.hour = time_set_hours;
			system_time.min = time_set_minutes;
			rtc_init();
			_delay_ms(1000);

			state = DEFAULT;
		}

		if (state == ALARM_TONE)
			display_tone();
		if (state == ALARM_TIME) {
			display_alarm_time();
			set_the_alarm();
			alarm_set_position = 1;
			display_alarm_time();
			set_the_alarm();
			display_alarm_time();
			_delay_ms(1000);
			state = DEFAULT;
			alarm_set_position = 0;
			finish_alarm();
		}
		
		if (alarm_on) {
			nr_rings--;
			display_ringing();
			ring_alarm();
			if (nr_rings == 0) {
				state = DEFAULT;
				PORTD &= ~(1 << PD7);
				PORTB |= (1 << PB5);
				alarm_on = 0;
				alarms[ringing_alarm_num].time.min += 1;
				nr_rings = 3;
			}
		}
		_delay_ms(100);
	}
	
	return 0;
}
